
package gs.html;

import java.sql.SQLException;

import java.util.Enumeration;
import java.util.Hashtable;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;

/**
 * Servlets that list the servlets in this virtual host
 * and their init parameters.
 *
 * @author Francis J. Lacoste
 */
public class Parser {

  /**
   * Retrieves the HTML file and replaces the variables with their values.
   * This version of the method does not take any record parser. Record
   * parsers handle &lt;record&gt; tags in the HTML templates.
   *
   * @exception  ServletException if there is any sort of Exception
   *
   */
  public static void parseHtml
  (
   String templateFile, 
   Hashtable fields, 
   ServletOutputStream out
    )
       throws IOException, ServletException, SQLException
  {
    parseHtml(templateFile, fields, out, null);
  }
    

  /**
   * Retrieves the HTML file and replaces the variables with their values.
   * This version of the method <b>requires</b> a record parser. Record
   * parsers handle &lt;record&gt; tags in the HTML templates.
   *
   * @exception  ServletException if there is any sort of Exception
   *
   */
  public static void parseHtml
  (
   String templateFile, 
   Hashtable fields, 
   ServletOutputStream out,
   RecordParsable rparser
    )
       throws IOException, ServletException, SQLException
  {
    //response.setContentType("text/html");
    //ServletOutputStream out = response.getOutputStream();
    //String filename = request.getPathTranslated();

    File htmlfile = new File( templateFile );

    if(!htmlfile.isFile()) 
      {
	throw new IOException("Could not open the file "+ templateFile +",");
      } else {
	
	BufferedReader lines = 
	  new BufferedReader
	  (new InputStreamReader
	   (new FileInputStream( htmlfile )));
	
	String aline, record="";
	int pos;
	boolean recordCollect = false;
	while ( ( aline = lines.readLine() ) != null )
	  {
	    if(!recordCollect)
	      {
		pos = aline.indexOf("<record>");
		if(pos >= 0 ) // this line needs a special treatment b/of the <record> tag.
		  {
		    if(pos>0) // there must be leading characters, so let parseLine handle those.
		      parseLine(fields, aline.substring(0, pos), out);
		    
		    aline = aline.substring(pos+8); // trim the leading chars, as well as <record> .
		    recordCollect = true;
		  }
		else // no record tag, just continue.
		  Parser.parseLine(fields, aline, out);
	      }

	    if(recordCollect)
	      {
		pos = aline.indexOf("</record>");
		if(pos >= 0) // <record> tag is closed.
		  {
		    record += aline.substring(0, pos); // code between tags are captured.
		    
		    // put the record treatment here!!!

		    if(rparser != null) // caller method may not provide a record parser!
		      while( rparser.handleRecord(fields) )
			parseLine(fields, record, out); 

		    // put the record treatment here!!!
		    
		    aline = aline.substring(pos+9);   // trim the closing record tag.
		    Parser.parseLine(fields, aline, out); // print the rest of the line.
		    recordCollect = false;
		  }
		else
		  record += "\n"+aline; // tag is not closed and start collecting
	      }
	  }
      }
  }
  
  public static void parseLine
  (Hashtable fields,
   String aline,
   ServletOutputStream out
    )
       throws IOException, ServletException
  {
    int pos = aline.indexOf("$"); // each variable must start with '$'.
    while (pos >= 0 && aline.length()>1)
      {
	if( pos != 0 ) 
	  {
	    if( aline.charAt(pos-1) == '\\' )     // escape character found!
	      aline = aline.substring(0, pos-1) + aline.substring(pos);
	    out.print( aline.substring(0, pos) ); // replace esc+$ w/ $
	    aline = aline.substring(pos);         // and print the heading part.
	    // continue w/ the rest.
	  } 
	// first char after $ must be an identifier starter usu. alphabetic chars.
	// we leave the job to Java.
	else if( Character.isUnicodeIdentifierStart(aline.charAt(1)) )
	  aline = Parser.parseVar( fields, aline ); 		                                           
	else 
	  { // if it is not a starter, then print $ and continue w/ the rest.
	    out.print( aline.substring(0, 1) );
	    aline = aline.substring(1);
	  }
	pos = aline.indexOf("$");
      }
    out.println( aline );
  }
  
  public static String parseVar
  (
   Hashtable fields, 
   String aline
    )
  {
    if(aline.length() > 1) 
      {
	int pos=1;
	String word="";
	// before coming here, we made sure that there is at least one char
	// which can make us an identifier.
	while( Character.isUnicodeIdentifierPart( aline.charAt(pos) ) ) 
	  {
	    word += aline.charAt(pos);
	    pos++;
	  }

	if( !word.equals("") )
	  {
	    String value = (String)fields.get(word);
	    aline = value + aline.substring(pos);
	  }
      }
    return aline;
  }
}
